# ----------------------------------------------------------------------------
# DTMF-Commands
# ----------------------------------------------------------------------------
# DTMF-Data-File-Format (the Index file)
# DATAFORMAT : CODE1    
#              CODE1    FileName1
#              CODE1    FileName2
#              CODE2    
#              CODE2    FileName2
# CODE is exact 8 Bytes long (filled up with Spaces!) + one Space as delimiter
#
# this is internally coded as follows
#
# 1. only CodeEntry selected (watch the special display)
#         HINT != "" FILE == ""
# 2. only FileEntry selected
#         HINT == "" FILE != ""
# 3. Code&FileEntry selected
#         HINT != "" FILE != ""
# ----------------------------------------------------------------------------


# ----------------------------------------------------------------------------
# Update the directory-listbox for DTMF

proc do_DTMF_update {} {
  global curr_dir curr_file index_db

  if ![file readable $curr_dir] {log_entry "Can't read $curr_dir" ; return 0}
  update_start
  read_DTMF
  set last_hint ""
  set hint_files ""
  foreach i $index_db {
    set hint "[lindex $i 0]        "
    set hint "[string range $hint 0 7]"
    if { "$last_hint" == "" } { set last_hint "$hint" }
    if { "$last_hint" == "$hint" } {
      if { [lindex $i 1] != "" } {
        if { "$hint_files" != "" } { set hint_files "$hint_files, " }
        set hint_files "$hint_files[lindex $i 1]"
      }
    } else {
#.....a new hint was found. The last one is finished.
      set dotted_code [string range "[string trim $last_hint]........" 0 7]
      set file_text "$dotted_code: $hint_files"
      insert_list "" "$file_text" "$last_hint"
      set hint_files ""
    }
    set last_hint "$hint"
  }
#.last entry
  set dotted_code [string range "[string trim $last_hint]........" 0 7]
  set file_text "$dotted_code: $hint_files"
  insert_list "" "$file_text" "$last_hint"

  insert_list "" "messages:" "" 

#.append all filenames detected by hint == ""
  set files [lsort [glob -nocomplain $curr_dir/$curr_file]]
  foreach f $files {
    if ![file isdirectory $f] {
      set file_name "[file tail $f]"
#.....don't show file 'Index' 
      if { $file_name != "Index" } {
        set file_text "          $file_name"
        insert_list "$file_name" "$file_text" ""
      }
    } 
  }

  update_end
}


# ----------------------------------------------------------------------------
# All selected file entries will be toggled for all selected codes.
# If no code entry but file entries are selected, a private DTMF code
# is requested and added.

proc do_DTMF_activate {} {
  global file_list hint_list index_db DTMF_file curr_mode n2

  set items [listbox_selection]
  if { $items == "" } {return 0}
  if ![file writable $DTMF_file] \
     {log_entry "Activate: can't write to $DTMF_file" ; return 0}
  set active [listbox_active]
  foreach line $items {
    set n [lindex $file_list $line]
#...look for selected file entries
    if {   [lindex $hint_list $line] == "" \
        && [lindex $file_list $line] != "" } {
      set code_found 0
      foreach line2 $items {
        set n2 [lindex $hint_list $line2]
#.......look for selected code entries
        if {   [lindex $file_list $line2] == "" \
            && [lindex $hint_list $line2] != "" } {
          set code_found 1
#.........is this file already active for the selected code?
          set db_line [lsearch -exact $index_db "$n2 $n"]
          if { -1 == $db_line } {
#...........no: activate it
            log_entry "$n: Activating file for code [string trim $n2]"
            lappend index_db "$n2 $n"
	  } else {
#...........yes: deactivate it
            log_entry "$n: Deactivating file for code [string trim $n2]"
            DTMF_delete "$n2 $n"
          }
        }
      }
#.....if no code entry was found: request a new code
      if { $code_found == 0 } {
        set n2 ""
        if { ![interact entry Activate n2] } {
#.........delete all non_numbers:
          regsub -all "\[^0-9\]" "$n2" "" n2
          if { "$n2" != "" } {
            set n2 [string range "$n2        " 0 7]
#...........is this a new code?
            if { -1 == [lsearch -exact $hint_list $n2] } {
              log_entry "[string trim $n2]: Creating code"
	    }
#...........is this code already activated for the file?
            set db_line [lsearch -exact $index_db "$n2 $n"]
            if { -1 == $db_line } {
#.............no: activate it
              log_entry "$n: Activating file for code [string trim $n2]"
              lappend index_db "$n2 $n"
            } else {
#.............yes: deactivate it
              log_entry "$n: Deactivating file for code [string trim $n2]"
              DTMF_delete "$n2 $n"
            }
          }
        }
      }
    }
  }
  write_DTMF
  do_mmail_mod $curr_mode
} 


# ----------------------------------------------------------------------------
# delete an entry from the index database

proc DTMF_delete {entry} {
  global index_db

  set i [lsearch -exact "$index_db" "$entry"]
  if { $i < 0 } {return 0}
  if { $i > 0 } {
    if { $i == [expr [llength $index_db] - 1] } {
      set index_db [lrange $index_db 0 [expr $i - 1]]
    } else {
      set index_db [concat [lrange $index_db 0 [expr $i - 1]] \
                           [lrange $index_db [expr $i + 1] end]]
    }
  } else {
    if { [llength $index_db] == 1 } {set index_db ""} \
    else {set index_db [lrange $index_db [expr $i + 1] end]}
  }
}


# ----------------------------------------------------------------------------
# Read the DTMF file

proc read_DTMF {} {
  global curr_dir index_db text DTMF_file

  set index_db {}
  set error 0
  if { ![catch {open "$DTMF_file" r} file_id] } {
    while {![eof $file_id]} {
      gets $file_id line
      if { [file exists $curr_dir/[lindex $line 1]] } {
        if { $line != "" } {
          lappend index_db "$line"
        }
      } else {set error 1}
    }
    catch {close $file_id}
  }
#.here a correction should be done!
#.FIRST add a Line for each Code without a Filename!
#.SECOND remove all equal lines !
#.this is braindamaged but it works ;-()
#.and all other function trust this routine
  foreach line "$index_db" {
    set blanked_code [string range "[lindex $line 0]        " 0 7]
    lappend index_db "$blanked_code "
  }
  set correction 1
  while { $correction == 1 } {
    set correction 0
    foreach line "$index_db" {
      set first [lsearch -exact "$index_db" "$line"]
      incr first
      if { [llength "$index_db"] > $first } {
        if { [lsearch -exact [lrange "$index_db" $first end] "$line"] > -1 } {
          DTMF_delete "$line"
          set correction 1
          if { [lindex "$line" 1] != "" } {set error 1} 
          break
        }
      }
    }
  }
  if $error {
    log_entry "WARNING: Corrected errors in DTMF_Index"
  }
  set index_db [lsort "$index_db"]
  write_DTMF
}


# ----------------------------------------------------------------------------
# Write the DTMF file

proc write_DTMF {} {
  global index_db DTMF_file

  if { [catch {open $DTMF_file w} file_id] } {
    log_entry "Can't write to $DTMF_file"
  } else {
    foreach entry "$index_db" {
      if { "$entry" != "" } {puts $file_id "$entry"}
    }
    catch {close $file_id}
  }
}


# ----------------------------------------------------------------------------
# Rename DTMF entries.
# Files and/or Codes can be renamed.
# A correction of the Index file will be done.
# Some selections are ignored since there is no need for an action.

proc do_DTMF_rename {} {
  global file_list hint_list index_db DTMF_file
  global curr_mode newname newcode curr_dir text

  set items [listbox_selection]
  if { $items == "" } {return 0}
  if ![file writable $DTMF_file] \
     {log_entry "Rename: can't write to $DTMF_file" ; return 0}
  if ![file writable $curr_dir] \
     {log_entry "Rename: can't write to $curr_dir" ; return 0} 
  set active [listbox_active]
  foreach line $items {
#...look for file entries
    if {   [lindex $hint_list $line] == "" \
        && [lindex $file_list $line] != "" } {
      set newname [lindex $file_list $line]
      set oldname "$newname"
      if { ![interact entry Rename newname] && "$newname" != "" } {
        if { "$newname" == "Index" } {
          log_entry "Rename: Can't use reserved name '$newname'"
        } else {
#.........replace spaces with underscores:
          regsub -all " " "$newname" "_" newname
          if { "$oldname" != "$newname" } {
#...........Check if there is already a file with this name
            if [file exists "$curr_dir/$newname"] {
              set text "Overwrite existing file $newname ?"
              set do_it ![interact label OK text]
              if $do_it {set overwrite 1}
            } else {set do_it 1 ; set overwrite 0}
            if $do_it {
              log_entry "$oldname: Renaming file"
              set command "mv -f \"$curr_dir/$oldname\" \"$curr_dir/$newname\""
              if [catch "eval exec $command"] {
                log_entry "$oldname: Renaming file -- failed"
              } else {
#...............renaming done
#...............now repair the Index File !
#...............Enter a CodeLine for all existing codes w/ the old filename
#...............with the new filename and remove the old one
                foreach line2 "$index_db" {
                  set linecode [lindex $line2 0]
                  set linename [lindex $line2 1]
                  if { "$linename" == "$oldname" } {
                    DTMF_delete "$line2"
                    set blanked_code [string range "$linecode        " 0 7]
                    lappend index_db "$blanked_code $newname"
                  }
                }                    
              }
            }
          }
        }
      }
    } else {
#.....look for code entries
      if {   [lindex $file_list $line] == "" \
          && [lindex $hint_list $line] != "" } {
        set newcode [string trim [lindex $hint_list $line]]
        set oldcode "$newcode"
        if { ![interact entry Rename newcode] }  {
#.........delete all non_numbers:
          regsub -all "\[^0-9\]" "$newcode" "" newcode
          if { "$newcode" != "" } {
            log_entry "$oldcode: Renaming code to $newcode"
            foreach line2 "$index_db" {
              if { "[string trim [lindex $line2 0]]" == "$oldcode" } {
                DTMF_delete "$line2"
                set blanked_code [string range "$newcode        " 0 7]
                lappend index_db "$blanked_code [lindex $line2 1]"
              }
            }
          }
        }
      }
    }
  }
  write_DTMF
  do_mmail_mod $curr_mode
}


# ----------------------------------------------------------------------------
# Delete DTMF entries.
# Files and/or Codes can be deleted.
# A correction of the Index file will be done.
# Some selections are ignored since there is no need for an action.

proc do_DTMF_remove {} {
  global file_list hint_list index_db DTMF_file
  global curr_mode newname curr_dir text

  set items [listbox_selection]
  if { $items == "" } {return 0}
  if ![file writable $DTMF_file] \
     {log_entry "Rename: can't write to $DTMF_file" ; return 0}
  if ![file writable $curr_dir] \
     {log_entry "Rename: can't write to $curr_dir" ; return 0} 
  set active [listbox_active]
  foreach line $items {
#...look for file entries
    if {   [lindex $hint_list $line] == "" \
        && [lindex $file_list $line] != "" } {
      set filename [lindex $file_list $line]
      set text "Delete selected file: $filename ?"
      set ret [interact label Delete text]
      if !$ret {
        log_entry "$filename: Removing file"
        catch "eval exec rm -f $curr_dir/$filename >/dev/null"
#.......now remove the file from all associated codes
        set correction 1
        while { $correction == 1 } {
          set correction 0
          foreach line2 "$index_db" {
            if { "$filename" == "[string trim [lindex $line2 1]]" } {
              DTMF_delete "$line2"
              set correction 1
              break
            }
          }
        }
      }
    } else {
#.....look for code entries
      if {   [lindex $file_list $line] == "" \
          && [lindex $hint_list $line] != "" } {
        set code [string trim [lindex $hint_list $line]]
        set text "Delete selected code: $code ?"
        set ret [interact label Delete text]
        if !$ret {
          log_entry "$code: Removing code"
          set correction 1
          while { $correction == 1 } {
            set correction 0
            foreach line2 "$index_db" {
              if { "$code" == "[string trim [lindex $line2 0]]" } {
                DTMF_delete "$line2"
                set correction 1
                break
              }
            }
          }
        }
      }
    }
  }
  write_DTMF
  do_mmail_mod $curr_mode
}

